package htLuokat;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Scanner;

import fi.jyu.mit.ohj2.Mjonot;

/**
 *  @author Leevi Pohja(pohjalma@jyu.fi), Jaakko Juvonen(jtjuvone@jyu.fi)
 *  , Jaakko Jokisalo(jjjokisa@student.jyu.fi)
 * @version 2 Mar 2022
 *
 */


public class Huoneet {

    private static final int  MAX_Huoneet = 10; 
    private Huone[] huoneet;
    private int lkm = 0; 
    private boolean muutettu = false; 

    
    /**
     * @param args ei kaytossa 
     */
    public static void main(String[]args)  {
        Huoneet huoneet = new Huoneet();
        
        Huone keittio = new Huone("keittio");
        Huone vessa = new Huone("vessa");
        keittio.rekisteroi();
        vessa.rekisteroi();
        try {
            
            huoneet.lisaa(keittio);
            huoneet.lisaa(vessa);
            
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        keittio.tulosta(System.out);
        vessa.tulosta(System.out);
    }
    
    
    /**
     * @param huone huone 
     * @example
     * <pre name="test">
     * Huoneet t = new Huoneet();
     * Huone h = new Huone("vessa");
     * h.rekisteroi();
     * t.lisaa(h);
     * int i = t.gethuoneetlkm();
     * i === 1;
     * t.gethuone(0).equals(t.gethuoneIdP(h.getId()));
     * t.lisaa(h);
     * t.lisaa(h);
     * t.lisaa(h);
     * i = t.gethuoneetlkm();
     * i === 4;
     * </pre>
     */
    public void lisaa(Huone huone)  {
        if(huone == null) return;
        if(lkm >= huoneet.length) {
            Huone[] uusi = new Huone[huoneet.length * 2];
            for(int i = 0; i < huoneet.length; i++) {
                uusi[i] = huoneet[i];
            }
            huoneet = uusi;
        }
        huoneet[lkm] = huone;
        lkm++;
        muutettu = true;
    }
    
    /**
     * palauttaa viitteen huoneeseen
     * @param i mones huone halutaan
     * @return viite huoneeseen
     * @throws IndexOutOfBoundsException jos i ei sallitulla alueella
     * @example
     * <pre name="test">
     * Huoneet t = new Huoneet();
     * Huone h = new Huone("vessa");
     * h.rekisteroi();
     * t.lisaa(h);
     * Huone h1 = new Huone("olohuone");
     * h1.rekisteroi();
     * t.lisaa(h1);
     * t.anna(0).equals(h);
     * t.anna(1).equals(h1);
     * t.anna(8); #THROWS IndexOutOfBoundsException
     * </pre>
     */
    public Huone anna(int i) throws IndexOutOfBoundsException {
        if(i < 0 || lkm <= i)
        throw new IndexOutOfBoundsException("Laiton indeksi" + i);
        return huoneet[i];
    }
    
    
    /**
     * Parametriton muodostaja
     */
    public Huoneet() {
        huoneet = new Huone[MAX_Huoneet];
    }
    
    
    /**
     * @return Huoneiden lkm
     */
    public int gethuoneetlkm() {
        return lkm;
    }
    
    
    /**
     * @param i huoneen indeksi
     * @return huone huone i indeksissä
     */
    public Huone gethuone(int i ) {
        return huoneet[i];
    }
    
    
    /**
     * haetaan huone id:n perusteella
     * @param id huoneen indeksi
     * @return huone
     * Huoneet h = new Huoneet();
     * Huone uusi = new Huone("kellari");
     * uusi.rekisteroi();
     * h.lisaa(uusi);
     * Huone huone = gethuoneIdP(uusi.getId());
     * huone.equals(uusi);
     * 
     */
    public Huone gethuoneIdP(int id ) {
        for(int i = 0; i< huoneet.length; i++) {
            if(huoneet[i].getId() == id) return huoneet[i];
        }
        return null;
    }
    
    
    /**
     * poistetaan huone 
     * @param poistettava poistettava huone
     */
    public void poista(Huone poistettava){
        if(poistettava == null) return;
        int id = poistettava.getId();
        int muutos = 0;
        Huone[] uusi = new Huone[huoneet.length];
       for(int i = 0, j = 0; i < this.lkm; i++) {
           if(id != huoneet[i].getId()) {
               uusi[j++] = huoneet[i];
           }
           else {
               muutettu = true;
               muutos = 1;
           }
       }
       huoneet = uusi;
       if(muutos == 1)this.lkm--;
       tallenna();
    }
    
    
    /**
     * Luetaan tiedostosta kayttajat 
     */
    public void luetiedostosta() {
     String tied = "huoneet.dat";
     
     try (Scanner fi = new Scanner(new FileInputStream(new File(tied)))) { // Jotta UTF8/ISO-8859 toimii
         while ( fi.hasNext() ) {
             try {
                 String rivi = fi.nextLine();
                 StringBuilder erotus = new StringBuilder();
                 erotus.append(rivi);
                 String nimi = Mjonot.erota(erotus, '|');
                 int id = Mjonot.erotaInt(erotus, -1);
                 Huone uusi = new Huone(nimi, id);
                 this.lisaa(uusi);   
             } catch (NumberFormatException ex) {
                 // Hylätään
             }
         }
     } catch (FileNotFoundException ex) {
         System.err.println("Tiedosto ei aukea! " + ex.getMessage());
         return;
     }
   }
    
    
    /**
     * tallennetaan kayttajat
     */
    public void tallenna(){
     if(muutettu == false) return;
     String tied = "huoneet.dat";  
     File t = new File(tied);
     t.delete();
     try (PrintStream fo = new PrintStream(new FileOutputStream(tied, true))) {
         for(int i = 0; i < this.lkm; i++) {
             fo.println(huoneet[i].getNimi() +"|"+ huoneet[i].getId());  
         }
       
         } catch (FileNotFoundException ex) {
             System.err.println("Tiedosto ei aukea: " + ex.getMessage());
         }
     }

    
    /**
     * Muokaa valitun objekti nimeä
     *   
     * @param muutos Muutettu nimi
     * @param muokattu Muokattu objekti
     */
    public void muokkaaNimea(String muutos, Object muokattu) {
        Huone h = (Huone) muokattu;
        h.setNimi(muutos);
        muutettu = true;     
    }
}
