package htLuokat;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Scanner;
import fi.jyu.mit.ohj2.Mjonot;

/**
 *  @author Leevi Pohja(pohjalma@jyu.fi), Jaakko Juvonen(jtjuvone@jyu.fi)
 *  , Jaakko Jokisalo(jjjokisa@student.jyu.fi)
 * @version 2 Mar 2022
 */
public class Liimataulut {
 
    
    private static final int  MAX_Liimataulut = 50; 
    private Liimataulu[] alkiot = new Liimataulu[MAX_Liimataulut];
    int seuraavaNro = 0;
    int lkm = 0;
    private boolean muutettu = false;

    
    /**
     *  
     */
    public Liimataulut() {
     
    }
    
    
    /**
     * @return lukumaaran 
     */
    public int getLkm() {
        return this.lkm;
    }
    
    
    /**
     * Poistetaan liimataulusta alkio 
     * @param hid huoneen id
     * @param tid tehtavan id 
     */
    public void poista(int hid, int tid){
        Liimataulu[] uusi = new Liimataulu[alkiot.length];
       
        int k = 0;
       for(int i = 0; i < this.lkm; i++) {
           if(alkiot[i].getHId() != hid || alkiot[i].getTId() != tid) {
               uusi[k++] = alkiot[i];
              
           }
           else {
               muutettu = true;
           }
           
       }
       alkiot = uusi;
       if (muutettu == true)lkm = k;
    }
    
    
    /**
     * Lisätään Liimataulut taulukkoon uusi olio
     * @param alkio lisättävä Liimataulut-olio
 * @example
 * <pre name="test"> 
 *  Liimataulut test = new Liimataulut();
 *  
 *  
 *  Liimataulu testi = new Liimataulu(1,1);
 *  for(int i = 0; i < 15; i++) test.lisaa(testi);
 *  test.getLkm()===15; 
 * </pre>
     */
    public void lisaa(Liimataulu alkio) {
        if(seuraavaNro >= alkiot.length -1) {
           Liimataulu[] uusi = new Liimataulu[alkiot.length * 2];
           for(int i = 0; i < this.lkm; i++) {
              uusi[i] = alkiot[i];
              
           }
           alkiot = uusi;
        }
        
        alkiot[lkm] = alkio;
        lkm++;
        seuraavaNro++;
        muutettu = true;
    }
    
    /**
     * Päivittää tehtävät
     * @param t tehtävä
     */
    public void paivitaT(Tehtava t) {
        if(t == null) return;
        int id = t.getId();
        int muutos = 0;
        Liimataulu [] uusi = new Liimataulu[alkiot.length];
        for(int i = 0, j = 0; i < this.lkm; i++) {
           if(alkiot[i].getTId() != id ) {
               uusi[j++] = alkiot[i];
           }
           else {
               muutos++;
           }
        }
        
        lkm = lkm - muutos;
        muutettu = true;
        tallenna();
    }
    
    
    /**
     * Poistetaan huone liimataulusta
     * @param h poistettu huone 
     */
    public void paivitaH(Huone h) {
        if(h == null) return;
        int id = h.getId();
        int muutos = 0;
        Liimataulu [] uusi = new Liimataulu[alkiot.length];
        for(int i = 0, j = 0; i < this.lkm; i++) {
           if(alkiot[i].getHId() != id ) {
               uusi[j++] = alkiot[i];
           }
           else {
               muutos++;
           }
        }
        
        lkm = lkm - muutos;
        muutettu = true;
        tallenna();
    }
    
    
    /**
     * Lisää tehätäviä huoneeseen
     * @param huone huone johon lisataan
     * @param t tehtavat taulukko
     */
    public void lisaaTehtavat(Huone huone, Tehtavat t) {
        for(int i = 0; i < t.getlkm(); i++) {
            Liimataulu uusi = new Liimataulu(huone.getId(), t.getTehtava(i).getId());
            lisaa(uusi);
        }
    
    }
    
    
    /**
     * @param str merkkijono 
     * @return numerot
     */
    public  int erotaSeuraavaNumero( String str) {                
        
        if(str == null || str.isEmpty()) return -1;
        
        for(char c : str.toCharArray()){
            if(Character.isDigit(c)){
               return Integer.parseInt(String.valueOf(c));
                
            }

        }
        
        return -1;
    }
    
    
    /**
     * Luetaan tiedostosta kayttajat 
     */
    public void luetiedostosta() {
     String tied = "Liimataulu.dat";
     
     try (Scanner fi = new Scanner(new FileInputStream(new File(tied)))) { // Jotta UTF8/ISO-8859 toimii
         while ( fi.hasNext() ) {
             try {
                 String rivi = fi.nextLine();
                 StringBuilder erotus = new StringBuilder();    
                 erotus.append(rivi); 
                 int hId = erotaSeuraavaNumero(erotus.toString());
                  Mjonot.erotaInt(erotus, 0);
                 int tId = erotaSeuraavaNumero(erotus.toString());
                 
                 
                 
                 Liimataulu uusi = new Liimataulu(hId, tId);
                
                    this.lisaa(uusi);
               
                 
             } catch (NumberFormatException ex) {
                 // Hylätään
             }
         }
     } catch (FileNotFoundException ex) {
         System.err.println("Tiedosto ei aukea! " + ex.getMessage());
         return;
     }
    }
    
    
    /**
     * Tallennetaan kayttajat
     */
    public void tallenna(){
      // TODO booleanin perusteella katsotaan onko muutoksia jotka tulisi tallentaa
        if (muutettu == false) return;
     String tied = "Liimataulu.dat";  
     File t = new File(tied);
     t.delete();
     int pituus =  this.getLkm();
     try (PrintStream fo = new PrintStream(new FileOutputStream(tied, true))) {
         for(int i = 0; i < pituus; i++) {
             fo.println(alkiot[i].getHId() +"|"+ alkiot[i].getTId());  
         }
       
         } catch (FileNotFoundException ex) {
             System.err.println("Tiedosto ei aukea: " + ex.getMessage());
         }
     muutettu = false;
     }

    
    /**
     * @param idH huone id 
     * @param idT tehtävä id
     * @return false jos ei ole kaytossa true jos on
     */
    public boolean onkoKaytossa(int idH, int idT) {
       for(int i = 0; i < this.lkm; i++) {
           if(idH == alkiot[i].getHId()) {
          int tehtavaId = alkiot[i].getTId();
          if( idT == tehtavaId) return true;    
           }
       }
        return false;
    }
}
    
    
