package htLuokat;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Scanner;

import fi.jyu.mit.ohj2.Mjonot;


/**
 *  @author Leevi Pohja(pohjalma@jyu.fi), Jaakko Juvonen(jtjuvone@jyu.fi)
 *  , Jaakko Jokisalo(jjjokisa@student.jyu.fi)
 * @version 2 Mar 2022
 * @example
 * <pre name="test">
    TehtavaLokit testi = new TehtavaLokit(); 
    Kayttaja testk = new Kayttaja("Testo-Timo", 5) ; 
    Huone testh = new Huone("Ghetto-gym", 9); 
    Tehtava testtht = new Tehtava("Roinaraivon jälkipyykki", 4); 
    TehtavaLoki testiMerkinta = new TehtavaLoki(testk, testh, testtht, "1999-08-30 klo 03:03:03"); 
        try {
        testi.lisaa(testiMerkinta);
    } catch (SailoException e) {
        
        e.printStackTrace();
    } 
    
    
    String s = testi.getSuoritus(testi, 0).toString(); 
    s.contentEquals("1999-08-30 klo 03:03:03 Testo-Timo Ghetto-gym Roinaraivon jälkipyykki") === true;
 * </pre>
 */


public class TehtavaLokit {
    
    private static final int MAX_SUORITUKSET = 1000;
    private TehtavaLoki[] suoritukset;
    private int lkm = 0;
    private boolean muutettu = false;


    /**
     * Haetaan suorituksien lukumäärä
     * @return suorituksien lukumäärä
     */
    public int getSLkm() {
        return lkm;
    }
    
    
    /**
     * Hakee tethävälokin indeksin paikalta
     * @param i indeksin paikka
     * @return tehtävälokin
     */
    public TehtavaLoki haeTehtavaLoki(int i ) {
        return suoritukset[i];
    }
    
    
    /**
     * Hakee suorituksen
     * @param i indeksin paikka
     * @return Suorituksen
     */
    public String[] haeSuoritus(int i) {
        if (lkm <  i) return null;
        String [] t = new String[4];

         t[0] = suoritukset[i].getKayttaja().toString();
         t[1] = suoritukset[i].getHuone().toString();
         t[2] = suoritukset[i].getTehtava().toString();
         t[3] = suoritukset[i].getKello();
      
       return t;
    }
    
    
    /**
     * @param suoritus suoritus
     * @throws SailoException heittää jos liikaa käytäjiä
     */
    public void lisaa(TehtavaLoki suoritus) throws SailoException {
        if(lkm >= suoritukset.length) throw new SailoException("Liikaa käyttäjiä!");
        suoritukset[lkm] = suoritus;
        lkm++;
        muutettu = true;
    }
    

    /**
     * Parametriton muodostaja
     */
    public TehtavaLokit() {
        suoritukset = new TehtavaLoki[MAX_SUORITUKSET]; 
        
    }
   
    
    /**
     * Luetaan tiedostosta kayttajat 
     * @param k kayttaja
     * @param h huone
     * @param t tehatava
     */
    public void luetiedostosta(Kayttajat k, Huoneet h, Tehtavat t) {
     String tied = "tehtavalog.dat";
     
     try (Scanner fi = new Scanner(new FileInputStream(new File(tied)))) { // Jotta UTF8/ISO-8859 toimii
         while ( fi.hasNext() ) {
             try {
                 String rivi = fi.nextLine();
                 StringBuilder erotus = new StringBuilder();
                 erotus.append(rivi);
                 
          
                 int kId = Mjonot.erota(erotus, '|', -1);
                 int hId = Mjonot.erota(erotus,'|' ,-1);             
                 int tId = Mjonot.erota(erotus, '|',-1);
                 String kello = erotus.toString();
                 
                 TehtavaLoki uusi = new TehtavaLoki(k.haekayttajaIdP(kId), h.gethuoneIdP(hId), t.getTehtavaIdP(tId), kello);
   
                    try {
                        this.lisaa(uusi);
                    } catch (SailoException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
               
                 
             } catch (NumberFormatException ex) {
                 // Hylätään
             }
         }
     } catch (FileNotFoundException ex) {
         System.err.println("Tiedosto ei aukea! " + ex.getMessage());
         return;
     }
    }
    
    
    /**
     * Poistetaan huone lokista 
     * @param h poistettu huone
     */
    public void paivitaH(Huone h) {
        
        if(h == null) return;
        int id = h.getId();
        int muutos = 0;
        TehtavaLoki [] uusi = new TehtavaLoki[suoritukset.length];
        for(int i = 0, j = 0; i < this.lkm; i++) {
            if(suoritukset[i].getHuoneId() != id) {
                uusi[j++] = suoritukset[i];
            }
            else {
                muutos++;
            }
            
        }
        lkm = lkm - muutos;
        muutettu = true;
        tallenna();
        
    }
    
    
    /**
     * Poistetaan tehtava 
     * @param t  poistettu tehtava
     */
    public void paivitaT(Tehtava t) {
        if(t == null) return;
        int id = t.getId();
        int muutos = 0;
        TehtavaLoki [] uusi = new TehtavaLoki[suoritukset.length];
        for(int i = 0, j = 0; i < this.lkm; i++) {
            if(suoritukset[i].getHuoneId() != id) {
                uusi[j++] = suoritukset[i];
            }
            else {
                muutos++;
            }
            
        }
        lkm = lkm - muutos;
        muutettu = true;
        tallenna();
    }
    
    
    /**
     * Poistamisen jälkeen päivitetään tehtäväloki niin ettei sinne jää poisttuja 
     * @param k poistettu kayttaja 
     */
    public void paivitaK(Kayttaja k) {
        if(k == null) return;
        int id = k.getId();
        int muutos = 0;
        TehtavaLoki [] uusi = new TehtavaLoki[suoritukset.length];
        for(int i = 0, j = 0; i < this.lkm; i++) {
            if(suoritukset[i].getKayttajaId() != id) {
                uusi[j++] = suoritukset[i];
            }
            else {
                muutos++;
            }
            
        }
        lkm = lkm - muutos;
        suoritukset = uusi;
        muutettu = true;
        tallenna();
    }
    
    
    /**
     * Tallennetaan kayttajat
     */
    public void tallenna(){
      // TODO booleanin perusteella katsotaan onko muutoksia jotka tulisi tallentaa
        if(muutettu == false) return;
     String tied = "tehtavalog.dat";  
     File t = new File(tied);
     t.delete();
     
     try (PrintStream fo = new PrintStream(new FileOutputStream(tied, true))) {
         for(int i = 0; i < this.lkm; i++) {
             fo.println(suoritukset[i].getKayttajaId() +"|"+ suoritukset[i].getHuoneId()+"|"+ suoritukset[i].getTehtavaId()+"|"+ suoritukset[i].getKellonaika() );  
         }
       
         } catch (FileNotFoundException ex) {
             System.err.println("Tiedosto ei aukea: " + ex.getMessage());
         }
     }

    
    /**
     * @param testi loki
     * @param i mones
     * @return suoritus
     */
    public TehtavaLoki getSuoritus(TehtavaLokit testi, int i) {
        // TODO Auto-generated method stub
        return testi.suoritukset[i];
    }    
}
