package htLuokat;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Scanner;

import fi.jyu.mit.ohj2.Mjonot;

/**
 *  @author Leevi Pohja(pohjalma@jyu.fi), Jaakko Juvonen(jtjuvone@jyu.fi)
 *  , Jaakko Jokisalo(jjjokisa@student.jyu.fi)
 * @version 2 Mar 2022
 *
 */
public class Tehtavat {
    private static final int  MAX_TEHTAVAT = 15; 
    private Tehtava[] kaikkiTehtavat;
    private int lkm = 0; 
    private boolean muutettu = false;
    
    
    /**
     * @param args ei kaytossa 
     */
    public static void main(String[]args)  {
        Tehtavat tehtavat = new Tehtavat();
        Tehtava imurointi = new Tehtava("Imurointi");
        imurointi.rekisteroi();
        imurointi.tulosta(System.out);
        
        Tehtava luuttuaminen = new Tehtava("Luuttuaminen");
        luuttuaminen.rekisteroi();
        luuttuaminen.tulosta(System.out);
        
        try {
            tehtavat.lisaa(imurointi);
            tehtavat.lisaa(luuttuaminen);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    
    
    /**
     * Lisää tehtävän
     * @param tehtava tehtava 
     * @example
     * <pre name="test">
     * Tehtavat t = new Tehtavat();
     * Tehtava j = new Tehtava("imurointi");
     * j.rekisteroi();
     * t.lisaa(j);
     * t.getTehtava(0).equals(j);
     * </pre>
     */
    public void lisaa(Tehtava tehtava) {
        if(lkm >= kaikkiTehtavat.length) {
            Tehtava [] uusi = new Tehtava[kaikkiTehtavat.length * 2];
            for(int i = 0; i < kaikkiTehtavat.length; i++) {
                uusi[i] = kaikkiTehtavat[i];
            }
            kaikkiTehtavat = uusi;
        }
        kaikkiTehtavat[lkm] = tehtava;
        lkm++;
        muutettu = true;
    }
    
    
    /**
     * palauttaa viitteen huoneeseen
     * @param i mones huone halutaan
     * @return viite huoneeseen
     * @throws IndexOutOfBoundsException jos i ei sallitulla alueella
     */
    public Tehtava anna(int i) throws IndexOutOfBoundsException {
        if(i < 0 || lkm <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi" + i);
        return kaikkiTehtavat[i];
    }
    
    
    /**
     * Parametriton muodostaja
     */
    public Tehtavat() {
        kaikkiTehtavat = new Tehtava[MAX_TEHTAVAT];
    }
    
    
    /**
     * @return lkm
     */
    public int getlkm() {
        
        return lkm;
    }
    
    
    /**
     * @param i indeksi
     * @return tehtava
     */
    public Tehtava getTehtava(int i) {
        
        return kaikkiTehtavat[i];
    }
    
    
    /**
     * Poistetaan tehtava 
     * @param poistettava poistettava olio 
     */
    public void poista(Tehtava poistettava) {
        if(poistettava == null) return;
        int id = poistettava.getId();
        int muutos = 0;
        Tehtava [] uusi = new Tehtava[kaikkiTehtavat.length];
        for(int i = 0, j = 0; i < this.lkm; i++) {
            if(kaikkiTehtavat[i].getId() != id) {
                uusi[j++] = kaikkiTehtavat[i];
            }
            else {
                muutettu = true;
                muutos = 1;
            }
            
        }
        
        kaikkiTehtavat = uusi;
        if(muutos == 1)this.lkm--;
        tallenna();
    }
    
    
    /**
     * @param id tehtava id
     * @return tehtava jolla on sama id
     */
    public Tehtava getTehtavaIdP(int id) {
        for(int i = 0; i < lkm; i++) {
        if(kaikkiTehtavat[i].getId() == id) return kaikkiTehtavat[i];
        }
        return null;
    }
    
    
    /**
     * Luetaan tiedostosta kayttajat 
     */
    public void luetiedostosta() {
     String tied = "tehtavat.dat";
     
     try (Scanner fi = new Scanner(new FileInputStream(new File(tied)))) { // Jotta UTF8/ISO-8859 toimii
         while ( fi.hasNext() ) {
             try {
                 String rivi = fi.nextLine();
                 StringBuilder erotus = new StringBuilder();
                 erotus.append(rivi);
                 String nimi = Mjonot.erota(erotus, '|');
                 int id = Mjonot.erotaInt(erotus, -1);
                 Tehtava uusi = new Tehtava(nimi, id);
                 
                
                    this.lisaa(uusi);
               
                 
             } catch (NumberFormatException ex) {
                 // Hylätään
             }
         }
     } catch (FileNotFoundException ex) {
         System.err.println("Tiedosto ei aukea! " + ex.getMessage());
         return;
     }
    }
    
    
    /**
     * Tallennetaan kayttajat
     */
    public void tallenna(){
      // TODO booleanin perusteella katsotaan onko muutoksia jotka tulisi tallentaa
        if (muutettu == false) return;
     String tied = "tehtavat.dat";  
     File t = new File(tied);
     t.delete();
     
     try (PrintStream fo = new PrintStream(new FileOutputStream(tied, true))) {
         for(int i = 0; i < this.lkm; i++) {
             fo.println(kaikkiTehtavat[i].getNimi() +"|"+ kaikkiTehtavat[i].getId());  
         }
       
         } catch (FileNotFoundException ex) {
             System.err.println("Tiedosto ei aukea: " + ex.getMessage());
         }
     }
    
    
    /**
     * Muutetaan olion nimi
     * @param muutos nimi muokattuna 
     * @param muokattu olio jonka nimeä käsitellään
     */
    public void muokkaaNimea(String muutos, Object muokattu) {
        Tehtava t = (Tehtava) muokattu;
        t.setNimi(muutos);
        muutettu = true;       
    } 
}
