package htScene;

import java.awt.Desktop;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.StringAndObject;
import fi.jyu.mit.fxgui.StringGrid;
import htLuokat.Kayttaja;
import htLuokat.KotityoRekisteri;
import htLuokat.TehtavaLoki;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;

/**
 *  @author Leevi Pohja(pohjalma@jyu.fi), Jaakko Juvonen(jtjuvone@jyu.fi)
 *  , Jaakko Jokisalo(jjjokisa@student.jyu.fi)
 * @version 31.1.2022
 *
 */


public class KotityoGUIController implements Initializable {
   
    private KotityoRekisteri rekisteri = new KotityoRekisteri();
    private muokkaaController muokkaa;
    private Kayttaja valittu; 
    private KotityoGUIController sivu;
    @FXML private ComboBoxChooser<Kayttaja> kayttajaIkkuna;
    
    
    /**
     * Suorituksen lisäys
     */
    SuoritusController suoritus;
   
    
    /**
     * Alla FXML objekteja
     */
    @FXML
    private StringGrid<TehtavaLoki> suoritusIkkuna = new StringGrid<>();   
    
    @FXML private BorderPane poistosivu;
    
    @FXML private Button about;

    @FXML private Button help;
    
    @FXML private TextField etsi;
    
    @FXML void handleAbout() {
        Dialogs.showMessageDialog("Kotityöloki" + "\n" + "Tekijät:"+ "\n"+ "Jaakko Jokisalo "
    + "\n" + "Jaakko Juvonen" + "\n" + "Leevi Pohja " + "\n" + "Version 0.3");
        
    }

    @FXML void handleHelp() {
        avustus();
    }
    
    @FXML
    void naytaKayttajat() {
        kayttajaIkkuna.clear();
        for(int i = 0; i < rekisteri.kayttajienLkm(); i++) {
            kayttajaIkkuna.add(rekisteri.getjasen(i).toString(),rekisteri.getjasen(i));
        }

    }
    
    @FXML void handleEtsi() {
        etsi(etsi.getText());
    }
    
    /**
     * Alustus
     */
    public void alusta() {
    
        suoritusIkkuna.initTable("Kayttaja", "Alue", "Tehtävä", "Aika");
        
        int lkm = rekisteri.suorituksienlkm();

        for(int i = 0; i < lkm; i++) {
            String [] t = rekisteri.haeSuoritus(i); 
            suoritusIkkuna.add( t[0], t[1], t[2], t[3]);    
        }
         
        suoritusIkkuna.setColumnWidth(3, 155);
        suoritusIkkuna.setEditable(false);
        naytaKayttajat();
      
        
    }
    
    
  /**
   *  Avaa Lisää-ikkunan
   */
    @FXML void avaaLisaa() {
        try {
        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("suoritus.fxml"));
        Parent root1 = (Parent) fxmlLoader.load();
        Stage stage = new Stage();
        stage.setScene(new Scene(root1));
        stage.show(); 
        
         suoritus = (SuoritusController)fxmlLoader.getController();
         suoritus.setrekisteri(rekisteri);
         suoritus.alusta();
         if(kayttajaIkkuna.getSelectedObject() == null) suoritus.setKayttaja(valittu);
         else suoritus.setKayttaja(kayttajaIkkuna.getSelectedObject()); //
         suoritus.haehuoneet();
         suoritus.haetehtavat(); 
         suoritus.setTulostusalue(suoritusIkkuna);
        
    } catch (Exception e) {
        System.err.println(e.getMessage()); 
        } 
    }
   
    
    /**
     * Avaa Muokkaa-ikkunan
     */
    @FXML void avaaMuokkaa() {
        try {
            FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("muokkausView.fxml"));
            Parent root1 = (Parent) fxmlLoader.load();
            Stage stage = new Stage();
            stage.setScene(new Scene(root1));
            stage.show(); 
                 
           muokkaa = (muokkaaController)fxmlLoader.getController();
           muokkaa.setrekisteri(rekisteri);
           muokkaa.vieViite(sivu);
          
        } catch (Exception e) {
            System.err.println(e.getMessage()); 
            }   
        }
    
    
/**
 * Poistoikkunan kontrollit
 */
    @FXML private Button cancelPoista;

    @FXML private Button poistaOk;

    @FXML void handleCancel() {
        ModalController.closeStage(poistosivu);
    }

    @FXML
    void handlePoistaOk() {
        Dialogs.showMessageDialog("Tässä kohtaa poistetaan valittu kohde(ei osata vielä)");
        ModalController.closeStage(poistosivu);
    }

    
    /**
     * @return true
     */
    public boolean avaa() {
        return true;
    }
    
    
    /**
     * @param kayttaja = valittu käytttäjä
     */
    public void setKayttaja(Kayttaja kayttaja) {
        valittu = kayttaja;
        kayttajaIkkuna.setPromptText(valittu.getNimi());
    }
    
    
    /**
     * Avustus
     */
    private void avustus() {
        Desktop desktop = Desktop.getDesktop();
        try {
            URI uri = new URI("https://tim.jyu.fi/view/kurssit/tie/ohj2/2022k/ht/pohjalma%22");
            desktop.browse(uri);
        } catch (URISyntaxException e) {
            return;
        } catch (IOException e) {
            return;
        }
    }
    
    
    /**
     * Alustaa suoritusikkunan
     */
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
       alusta();
    }
    
    
    /**
     * @param rekisteri rekisteri
     */
    public void setrekisteri(KotityoRekisteri rekisteri) {
        this.rekisteri = rekisteri;
        alusta();
    } 
    
    
    /**
     * Päivittää näkyvät merkinnät hakuehdon perusteella
     * @param hakuehto hakupalkkiin kirjoitettu merkkijono
     */
    public void etsi(String hakuehto) {
        StringGrid<TehtavaLoki> etsiIkkuna = suoritusIkkuna;
        
        if(etsi.getText() == "") {
            alusta();
            return;
        }       
        
        etsiIkkuna.clear();
        etsiIkkuna.initTable("Kayttaja", "Alue", "Tehtävä", "Aika");
              
        int lkm = rekisteri.suorituksienlkm();

        for(int i = 0; i < lkm; i++) {
            String [] t = rekisteri.haeSuoritus(i); 
            for(int j = 0; j < t.length; j++)
                if(t[j].toLowerCase().contains(hakuehto.toLowerCase())) etsiIkkuna.add( t[0], t[1], t[2], t[3]);               
        }   
        etsiIkkuna.setColumnWidth(3, 155);
        etsiIkkuna.setEditable(false);
        etsiIkkuna.getObject();
    }

    
    /**
     * Vie viitteen
     */
    public void vieViite() {
        sivu = this;
    }
}
